# Copyright 2015 Timothy C Eichler (c) , All rights reserved.
# Restrictive license being used whereby the following apply.
# 1. Non-commercial use only
# 2. Cannot modify source-code for any purpose (cannot create derivative works)
# The full license can be viewed at the link below
# http://www.binpress.com/license/view/l/9dfa4dbfe85c336d16d1dd71a2e2cfb2

#!/usr/bin/env python3
import subprocess    
import time  
import os
import getpass

import psutil
from timstools import only_numerics

if os.name == 'nt':
    import win32gui
    import win32process
    import win32con

def process_exists(pid=None):
    """
    Evaluates a Pid Value defaults to the currently foucsed window
    against the current open programs,
    if there is a match returns the process name and pid
    otherwise returns None, None
    """
    if not pid:
        pid = current_pid()
    elif callable(pid):
        pid = pid()

    if pid and psutil.pid_exists(pid):
        pname = psutil.Process(pid).name()
        if os.name == 'nt':
            return os.path.splitext(pname)[0], pid
        return pname, pid
    return None, None


def get_window_id(): #used by xdotool, good idea to save as reference   #this is window id of currently focused window
    cmd=['xdotool','getactivewindow']        # Gives a unique window id which we will use to get our PID    #TBD this method meant to be better supposingly, trying it out if no errors we all good
    #~ cmd=['xdotool','getwindowfocus']        # Gives a unique window id which we will use to get our PID
    args = subprocess.Popen(cmd, stdout = subprocess.PIPE, stderr= subprocess.PIPE).communicate()
    #out=(args.stdout.read())
    args=str(args[0])                       # Just take the output not the errors from the tuple
    args=only_numerics(args)                # gives us only numerical numbers
    return args

def current_pid():
    if os.name == 'nt':
        hwnd = win32gui.GetForegroundWindow()
        _, pid = win32process.GetWindowThreadProcessId(hwnd)
    else:
        window_id = get_window_id()
        cmd=["xdotool","getwindowpid", window_id]   # Uses the Window Id to find the PID
        args = subprocess.Popen(cmd,stdout = subprocess.PIPE, stderr= subprocess.PIPE).communicate()
        pid = args[0].decode('utf-8')
        #~ pid = only_numerics(args)         # Our Pid of the Currently focused Window at time of function call
    try:
        return int(pid) 
    except (ValueError, TypeError): 
        return None

def move_active_window(x, y):
    """
    Moves the active window to a given position given the window_id and absolute co ordinates,
    --sync option auto passed in, will wait until actually moved before giving  control back to us

    will do nothing if the window is maximized
    """
    window_id = get_window_id()
    cmd=['xdotool','windowmove', window_id, str(x), str(y)]
    subprocess.Popen(cmd, stdout = subprocess.PIPE, stderr= subprocess.PIPE).communicate()

def get_active_window_pos():
    '''screen coordinates massaged so that movewindow command works to
    restore the window to the same position
    returns x, y
    '''
    # http://stackoverflow.com/questions/26050788/in-bash-on-ubuntu-14-04-unity-how-can-i-get-the-total-size-of-an-open-window-i/26060527#26060527
    cmd = ['xdotool','getactivewindow', 'getwindowgeometry']
    res = subprocess.Popen(cmd, stdout = subprocess.PIPE, stderr= subprocess.PIPE).communicate()
    stdout = res[0].decode('utf-8').splitlines()
    pos = stdout[1].split(':')[1].split(',')
    geo = stdout[2].split(':')[1].split('x')
    x, y = int(pos[0].strip()), int(pos[1].split('(')[0].strip())
    w, h = int(geo[0].strip()), int(geo[1].strip())

    # get the window decorations
    window_id = get_window_id()
    cmd = ['xprop', '_NET_FRAME_EXTENTS', '-id', window_id]
    res = subprocess.Popen(cmd, stdout = subprocess.PIPE, stderr= subprocess.PIPE).communicate()
    decos = res[0].decode('utf-8').split('=')[1].split(',')
    l, r = int(decos[0].strip()), int(decos[1].strip())
    t, b = int(decos[2].strip()), int(decos[3].strip())

    return x-l, y-t

from socket import gethostname

def listpid(toggle='basic'): # Add method to exclude elements from list
    '''list pids'''
    proc=psutil.process_iter()# evalute if its better to keep one instance of this or generate here?
    if toggle=='basic':
        host=gethostname()
        host2=os.getenv('HOME').split(sep='/'   )[-1]
        for row in proc:
            #~ DPRINT([row.ppid(),row.name(),host],'username,row.name,host')
            if row.username() in host or row.username() in host2:   #new psutil using grabing timeyyy and not alfa for username so host 2 is getting the timeyyy on UBUNTU  
                yield row.name(), row.ppid()
    elif toggle=='all':
        for row in proc:
            
            yield row.name(), row.ppid()
    elif toggle =='windows-basic':
        for row in proc:
            try:
                pname = psutil.Process(row.pid).name()
                pname = pname[:-4]#removiing .exe from end
                yield pname, row.pid
            except:
                pass#cannot get info on system proccessors, returns exception

    #~ for proc in psutil.process_iter():
        #~ try:
            #~ print (proc.pid, proc.username)
#~ 
        #~ except psutil.AccessDenied:
            #~ print("denied")
    #~ 

def show_window(value):
    if os.name == 'nt':
        HWND = value
        win32gui.ShowWindow(HWND, win32con.SW_RESTORE)
        win32gui.SetWindowPos(HWND,win32con.HWND_NOTOPMOST, 0, 0, 0,0, win32con.SWP_NOMOVE + win32con.SWP_NOSIZE )
        win32gui.SetWindowPos(HWND,win32con.HWND_TOPMOST, 0, 0, 0,0,win32con.SWP_NOMOVE + win32con.SWP_NOSIZE )
        win32gui.SetWindowPos(HWND,win32con.HWND_NOTOPMOST, 0, 0, 0,0,win32con.SWP_SHOWWINDOW + win32con.SWP_NOMOVE + win32con.SWP_NOSIZE)
        #~ win32gui.ShowWindow(hWnd,win32con.SW_SHOW); 
        #~ win32gui.BringWindowToTop(hWnd);
        #~ win32gui.SetForegroundWindow(hWnd);
        #http://stackoverflow.com/questions/6312627/windows-7-how-to-bring-a-window-to-the-front-no-matter-what-other-window-has-fo
        #http://timgolden.me.uk/pywin32-docs/win32gui__SetWindowPos_meth.html
        #~ //-- on Windows 7, this workaround brings window to top
        #~ win32gui.SetWindowPos(hWnd,HWND_NOTOPMOST,0,0,0,0, win32con.SWP_NOMOVE | win32con.SWP_NOSIZE);
        #~ win32gui.SetWindowPos(hWnd,HWND_TOPMOST,0,0,0,0,win32con.SWP_NOMOVE | win32con.SWP_NOSIZE);
        #~ win32gui.SetWindowPos(hWnd,HWND_NOTOPMOST,0,0,0,0,win32con.SWP_SHOWWINDOW | win32con.SWP_NOMOVE | win32con.SWP_NOSIZE);
    else:
        name = value
        cmd = ['xdotool','search','--name','--onlyvisible', name,'windowactivate']
        args = subprocess.Popen(cmd, stdout = subprocess.PIPE, stderr= subprocess.PIPE).communicate()
    
def get_hwnds_for_pid (pid):
    def callback (hwnd, hwnds):
        #~ if win32gui.IsWindowVisible (hwnd) and win32gui.IsWindowEnabled (hwnd):
        _, found_pid = win32process.GetWindowThreadProcessId (hwnd)
        if found_pid == pid:
            hwnds.append (hwnd)
        return True
    hwnds = []
    win32gui.EnumWindows (callback, hwnds)
    return hwnds

def get_active_title():
    '''returns the window title of the active window'''
    if os.name == 'posix':
        cmd = ['xdotool','getactivewindow','getwindowname']
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        title = proc.communicate()[0].decode('utf-8')
    else:
        raise NotImplementedError
    return title

def get_processes():
    '''returns process names owned by the user'''
    user = getpass.getuser()
    for proc in  psutil.process_iter():
        if proc.username() != user:
            continue
        pname = psutil.Process(proc.pid).name()
        if os.name == 'nt':
            pname = pname[:-4]          # removiing .exe from end
        yield pname

def get_titles():
    '''returns titles of all open windows'''
    if os.name == 'posix':
        for proc in get_processes():
            cmd = ['xdotool','search','--name', proc]
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            window_ids = proc.communicate()[0].decode('utf-8')
            if window_ids:
                for window_id in window_ids.split('\n'):
                    cmd = ['xdotool','getwindowname',window_id]
                    proc = subprocess.Popen(cmd,
                                        stdout=subprocess.PIPE,
                                        stderr=subprocess.PIPE)
                    title = proc.communicate()[0].decode('utf-8')
                    try:
                        if title[-1] == '\n':
                            title = title[:-1]
                        yield title
                    except IndexError:
                        pass
    else:
        raise NotImplementedError

if __name__ == '__main__':
    
    # from pprint import pprint
    # pprint(list(get_titles()))
    # print(get_titles())
    pass
    #~ foregroundWindow('Mozilla Firefox')


#im using xdo alot, wmctrl used by txkert       
#try xdotool http://www.semicomplete.com/projects/xdotool/xdotool.xhtml#window_commands
#replacement for xdo http://tomas.styblo.name/wmctrl/
    
    #~ pid_table = psutil.process_iter()
    #~ genPidL=pidList(pid_table)
    #~ pidList(pid_table)
    
    """                     # Test methods of host name
    import platform
    print(platform.node())
    import socket
    print(socket.gethostname())
    """

    





"""

  peasoup.util:  misc utility functions for peasoup

"""
import sys
import os

#  Since Peaoup will have
#  every invocation, we want as little overhead as possible when importing
#  the main module.  We therefore use a simple lazy-loading scheme for many
#  of our imports, built from the functions below.

def lazy_import(func):
    """Decorator for declaring a lazy import.

    This decorator turns a function into an object that will act as a lazy
    importer.  Whenever the object's attributes are accessed, the function
    is called and its return value used in place of the object.  So you
    can declare lazy imports like this:

        @lazy_import
        def socket():
            import socket
            return socket

    The name "socket" will then be bound to a transparent object proxy which
    will import the socket module upon first use.

    The syntax here is slightly more verbose than other lazy import recipes,
    but it's designed not to hide the actual "import" statements from tools
    like py2exe or grep.
    """
    try:
        f = sys._getframe(1)
    except Exception:
        namespace = None
    else:
        namespace = f.f_locals
    return _LazyImport(func.__name__,func,namespace)


class _LazyImport(object):
    """Class representing a lazy import."""

    def __init__(self,name,loader,namespace=None):
        self._esky_lazy_target = _LazyImport
        self._esky_lazy_name = name
        self._esky_lazy_loader = loader
        self._esky_lazy_namespace = namespace

    def _esky_lazy_load(self):
        if self._esky_lazy_target is _LazyImport:
            self._esky_lazy_target = self._esky_lazy_loader()
            ns = self._esky_lazy_namespace
            if ns is not None:
                try:
                    if ns[self._esky_lazy_name] is self:
                        ns[self._esky_lazy_name] = self._esky_lazy_target
                except KeyError:
                    pass

    def __getattribute__(self,attr):
        try:
            return object.__getattribute__(self,attr)
        except AttributeError:
            if self._esky_lazy_target is _LazyImport:
                self._esky_lazy_load()
            return getattr(self._esky_lazy_target,attr)

    def __bool__(self):
        if self._esky_lazy_target is _LazyImport:
            self._esky_lazy_load()
        return bool(self._esky_lazy_target)

def sftp_upload_window_size_set(srv,file, method_to_call='put'):
    '''
    sets config for uploading files with pysftp
    '''
    channel = srv.sftp_client.get_channel()
    channel.lock.acquire()
    channel.out_window_size += os.stat(file).st_size * 1.1   # bit more bytes incase packet loss
    channel.out_buffer_cv.notifyAll()
    channel.lock.release()




from time import gmtime, strftime
from contextlib import suppress
from base64 import b64decode as crypto
import codecs

from peasoup.util import lazy_import

@lazy_import
def traceback():
    import traceback
    return traceback

@lazy_import
def threading():
    import threading
    return threading

@lazy_import
def os():
    import os
    return os


import tkquick.gui.maker as maker
from peasoup.gui import uploader as gui_uploader
import tkinter.ttk as ttk
from peasoup.gui import uploader as gui_uploader

class LogUploader():
    def logexception(self, logger=None):
        '''
        calls exception method on a loger and prints the log to stdout
        logger is set in the cfg
        #Todo more details here on the cfg etc
        :param logger:
        :return:
        '''
        traceback.print_exc()
        if logger: 
            logger.exception('Unexpected runtime Error...')
        else:
            self.logger.exception('Unexpected runtime Error...')

    def upload_logs(self, release_singleton=True):
        '''
        uploads a log to a server using the method and gui specifed
        in self.pcfg
        singleton mode can be disabled so a new version can be restarted whille uploading oges
        on typicallin in the case of uploadnig after a crash or sys exit
        set self.cfg log_upload_interface to gui/cli/or background
        '''
        if release_singleton:
            self.release_singleton()

        def _upload():
            for log in self.get_logs():
                new_name = self._uniquename(log)
                self._upload(log, new_name)
                # Todo: keep log around for a few days
                self.delete_log(log)

        if self.pcfg['log_server_interface'] == 'gui':
            raise NotImplementedError
            threading.Thread(target=threadme)
            gui_uploader(threadme)
        elif self.pcfg['log_server_interface'] == 'cli':
            raise NotImplementedError
        elif self.pcfg['log_server_interface'] == 'background':
            _upload()

    def delete_log(self, log):
        '''check we don't delte anythin unintended'''
        if os.path.splitext(log)[-1] != '.log':
            raise Exception('File without .log was passed in for deletoin')
        with suppress(Exception):
            os.remove(log)


    def get_logs(self):
        '''returns logs from disk, requires .log extenstion'''
        folder = os.path.dirname(self.pcfg['log_file'])
        for path, dir, files in os.walk(folder):
            for file in files:
                if os.path.splitext(file)[-1] == '.log':
                    yield os.path.join(path, file)

    # TODO MAKE THIS TAKE THE TIME FROM THE LOG IMPORTANT
    def _uniquename(self, log):
        '''
        renames the log to ensure we get no clashes on the server
        subclass this to change the path etc'''
        return '{hostname} - {time}.log'.format(
                                hostname=os.getenv('USERNAME'),
                                time=strftime("%a, %d %b %Y %H-%M-%S", gmtime()))

    def _upload(self, log, new_name):
        if self.pcfg['log_server_method'] == 'sftp':
            import pysftp
            from peasoup.util import sftp_upload_window_size_set
            try:
                srv = pysftp.Connection(host=self.pcfg['log_server_host'],
                                        username=self.pcfg['log_server_username'],
                                        password=self.rot13(self.pcfg['log_server_password']),
                                        port=self.pcfg['log_server_port'])
                self.logger.info(
                        'Uploading log to server %s' % self.pcfg['log_server_host'])
                sftp_upload_window_size_set(srv, log)
                srv.put(log, new_name)
            except pysftp.ConnectionException:
                raise pysftp.ConnectionError('Could not connecto to log server')
            finally:
                with suppress(UnboundLocalError):
                    srv.close()
        else:
            # need to implement the gui uploaders...
            raise NotImplementedError

    def release_singleton(self):
        raise NotImplementedError('Please subclass')

    def rot13(self, encrypyted):
        ''' Very secure encryption (ceaser used it), apply it multiple times'''
        def random():
            # guarnteed random from fair dice roll
            return 4
        return codecs.encode(
                        codecs.encode(
                                crypto(encrypyted).decode(),
                                'rot_{amount}'.format(amount=int(52/random()))),
                        'rot_{salt}'.format(salt=int(52/random())))

def add_date(log):
    '''Userful for randomizing the name of a log'''
    return '{base} - {time}.log'.format(
			base=os.path.splitext(log)[0],
			time=strftime("%a, %d %b %Y %H-%M-%S", gmtime()))


# def handle_fatal_exception(
#         error,
#         restarter,
#         file,
#         host,
#         username,
#         password,
#         port,
#         pre_restarter='gui',
#         output=None):
#     '''
#     uploads a log to a server
#     displays a progress widget using tktiner
#     restart function is required in the case of the user wanting to restart
#     '''
#     def upload_log():
#         # this is run in a thread
#         nonlocal output
#         import pysftp
#         from time import gmtime, strftime
#         from timstools import sftp_upload_window_size_set
#         try:
#             srv = pysftp.Connection(host=host, username=username, password=password, port=port)
#             logging.info('Uploading log to server')
#             sftp_upload_window_size_set(srv, file)
#             if not output:
#                 output = os.getenv(
#                         'USERNAME') + '-' + strftime("%a, %d %b %Y %H-%M-%S", gmtime()) + '.log'
#             srv.put(file, output)
#         except pysftp.ConnectionException:
#             pass
#         finally:
#             with suppress(UnboundLocalError):
#                 srv.close()
#             tkinter_queue.put(root.quit)
#
#     def poll_tkinter_queue():
#         # tktiner after method not avaliable from other threads!,
#         # this runs functions put on the queue
#         try:
#             func = tkinter_queue.get(block=False)
#         except queue.Empty:
#             pass
#         else:
#             func()
#         root.after(100, poll_tkinter_queue)
#
#     import traceback
#     print()
#     traceback.print_exc()
#     logging.exception('Unexpected runtime Error Occured')
#     root = tk.Tk()
#     root.withdraw()
#
#     tkinter_queue = queue.Queue()
#     thread.start_new_thread(upload_log, ())
#     root.after(100, poll_tkinter_queue)
#     #~ ttk.Label(root, text='Uploading error logs! please wait...').pack()
#     #~ prog = ttk.Progressbar(root, mode='indeterminate', length=100)
#     #~ prog.pack()
#     #~ prog.start()
#     #~ maker.center_window(root, 100, 20)
#     #~ root.deiconify()
#     root.mainloop()
#
#     if pre_resstarter == 'gui':
#         if messagebox.askyesno(
#                 'Unexpected Error',
#                 ' Sorry for the inconvienience. Would you like to restart?',
#                 parent=root):
#
#             logging.debug('running restarter function')
#             restarter()
#     else:
#         print('TODO custom pre restart')
#         # pre_restarter()
#         # restart()



# Todo defil __all__ for the * imports
from peasoup.peasoup import *
from peasoup.pidutil import *
from peasoup.restart import Restarter
from peasoup.uploadlogs import add_date
from peasoup.errors import *



# Copyright 2015 Timothy C Eichler (c) , All rights reserved.
'''
see design spec
'''
import os
import sys
import platform
import time
import inspect
import logging

import appdirs
from timstools import ignored as suppress

from peasoup import pidutil       # Todo delete this after done integrating get pid
from peasoup.util import lazy_import
from peasoup.uploadlogs import LogUploader

@lazy_import
def json():
    import json
    return json

@lazy_import
def yaml():
    import yaml
    return yaml

@lazy_import
def esky():
    import esky
    return esky

if os.name == 'nt':
    import win32security
    import ntsecuritycon as con
    import win32api
    import pywintypes

PEASOUP_USER_DIR = 'autobots'
PEASOUP_CONFIG_FILE = 'config.json'

class CfgDict(dict):
    '''
    Because json is alwasy going to give us a normal dict we have to
    add some new mothods after the fact,
    This is a collection of methods for our config dict
    Pass in the old dict and wrap on some shiny new methods

    This is jus like a decorator but i'm not using the
    explict syntax on it #TODO
    '''
    def __init__(self, app, cfg):
        super().__init__(self, **cfg)
        self.app = app

    def __getattribute__(self, item):
        if item == 'save':
            return lambda: CfgDict.save(self)
        else:
            return object.__getattribute__(self, item)

    # TODO To be honest i have no F*** what is going on here but it works, how to remove this self.app state...
    def save(self):
        '''saves our config objet to file'''
        if self.app.cfg_mode == 'json':
            with open(self.app.cfg_file, 'w') as opened_file:
                json.dump(self.app.cfg, opened_file)
        else:
            with open(self.app.cfg_file, 'w')as opened_file:
                yaml.dump(self.app.cfg, opened_file)

class AppBuilder(LogUploader):

    '''
    Add functions to the dictionary -> self.shutdown_cleanup
    the functions will be run on cleanup, if you choose not, to use
    the shutdown function and use the check_if_open function you will
    need to run the functions in self.shutdown_cleanup on shutdown

    main_file is required to get the abspath of the script
    '''

    def __init__(self, main_file, pcfg_path=None, pcfg_name=None):
        logging.warning('PEASOUP pcfg_path needs to be finished coded')
        AppBuilder.main_file = main_file
        self.pcfg = self.get_pcfg()
        self.app_name = self.pcfg['app_name']
        self.shutdown_cleanup = {}
        self.start_time = time.time()

    @staticmethod
    def get_pcfg():
        '''
        sets up the config options by reading globals saved in peasoup/global.py
        as self.pcfg
        '''
        path = os.path.dirname(AppBuilder.main_file)
        file = os.path.join(path,
                            PEASOUP_USER_DIR,
                            PEASOUP_CONFIG_FILE)
        with open(file) as f:
            pcfg = json.load(f)
        return pcfg

    def create_cfg(self, cfg_file, defaults=None, mode='json'):
        '''
        set mode to json or yaml? probably remove this option..Todo

        Creates the config file for your app with default values
        The file will only be created if it doesn't exits

        also sets up the first_run attribute.

        also sets correct windows permissions

        you can add custom stuff to the config by doing
        app.cfg['fkdsfa'] = 'fdsaf'
        # todo auto save on change
        remember to call cfg.save()

        '''
        assert mode in ('json', 'yaml')
        self.cfg_mode = mode
        self.cfg_file = cfg_file
        try:
            self.cfg = CfgDict(app=self, cfg=self.load_cfg())
            logging.info('cfg file found : %s' % self.cfg_file)
        except FileNotFoundError:
            self.cfg = CfgDict(app=self, cfg={'first_run': True})
            with suppress(TypeError):
                self.cfg.update(defaults)
            self.cfg.save()
            set_windows_permissions(self.cfg_file)
            logging.info(
                'Created cfg file for first time!: %s' %
                self.cfg_file)

        if self._check_first_run():
            self.first_run = True
        else:
            self.first_run = False

    def load_cfg(self):
        '''loads our config object accessible via self.cfg'''
        if self.cfg_mode == 'json':
            with open(self.cfg_file) as opened_file:
                return json.load(opened_file)
        else:
            with open(self.cfg_file) as ymlfile:
                return yaml.safe_load(ymlfile)


    def _check_first_run(self):
        '''
        Checks if this is the first time our app is being run
        This is called automatically when using create_cfg()
        use then app.first_run attribute to check for True or False
        '''
        if self.cfg['first_run']:
            self.cfg['first_run'] = False
            self.cfg.save()
            return True
        else:
            return False

    def is_installed(self):
        '''If the program is fully installed returns 1
        Only invoked if the program is being run from Program Files     #Tbd make this more robust
        '''
        if os.name == 'nt':
            # tbd find a better way to tell if  a program is installed in
            # registry
            if 'Program Files' in os.getcwd():
                return 1
            else:
                return 0
        else:
            print('TBD LINUX CHECK FULLY INSTLALED APPTOOLS')
            return 0

    # http://stackoverflow.com/questions/16872448/c-sharp-write-and-read-appsettings-and-log-files-best-practice-uac-program-file
    def get_appdir(self, portable_path=None, folder=None, create=False):
        '''
        path = uac_bypass(file)

        This function will only operate when your program is installed
        check the is_installed function for details

        Moves working data to another folder. The idea is to get around
        security and uac on windows vista +

        returns cwd on linux, windows returns path with write
        access: C:\\ProgramData\\appname here

        if a file is passed in it will be appended to the path

        set create to true to create the file in the programData folder

        setting overwrite to True will silently overwrite, otherwise a
        FileExistsError is raised

        Setting overwrite to get, will get the file path instead of throwing an error

                                ---Background----
        If a user is using windows, Read write access is restriced in the
        Program Files directory without prompting for uac.

        We create a folder in c:\Program Data\ of the program name and
        save logging data etc there.

        This way the program doesnt need admin rights.

        '''
        if self.is_installed():
            appdir = appdirs.user_data_dir(self.app_name)
        elif portable_path:
            appdir = portable_path
            if not folder:
                folder = 'data^-^'
        else:
            appdir = os.getcwd()

        if folder:
            path = os.path.join(appdir, folder)
        else:
            path = appdir

        path = os.path.realpath(path)

        if create:
            os.makedirs(path, exist_ok=1)
        return path


    def init_file(self, path, file, create=False, overwrite=False):
        logging.info(
            'Uac Bypass - Attempting to Create: %s' %
            os.path.join(
                path,
                file))
        TO = os.path.join(path, file)
        if create:
            os.makedirs(path, exist_ok=1)
        if overwrite and overwrite != 'get':
            new_file = open(TO, 'w')
            new_file.close()
        elif not os.path.exists(TO):
            new_file = open(TO, 'w')
            new_file.close()
        elif overwrite != 'get':
            raise FileExistsError
        return os.path.join(path, file)


    def check_if_open(self, path=None, appdata=False, verbose=False):
        '''
        Allows only one version of the app to be open at a time.

        If you are calling create_cfg() before calling this,
        you don't need to give a path. Otherwise a file path must be
        given so we can save our file there.

        Set appdata to True to run uac_bypass on the path, otherwise
        leave it as False
        '''
        #~ To know if the system crashed, or if the prgram was exited smoothly
        #~ turn verbose to True and the function will return a named tuple  # TBD
        #~ if os.name == 'nt':
        #~ hwnd = int(self.root.wm_frame(),0)
        #~ #saving a hwnd reference so we can check if we still open later on
        #~ with open (self.check_file,'a') as f:
        #~ f.write(str(hwnd))
        #~ logging.info('adding hwnd to running info :'+str(hwnd))
        #~
        logging.info('Checking if our app is already Open')
        if not path and self.cfg:
            self._check_if_open_using_config()
        elif path:
            if appdata:
                file = path.split(os.sep)[-1]
                self.check_file = self.uac_bypass(file=file)
            else:
                self.check_file = path
            self._check_if_open_using_path()

        self.shutdown_cleanup['release_singleton'] = self.release_singleton

    def release_singleton(self):
        '''deletes the data that lets our program know if it is
        running as singleton when calling check_if_open,
        i.e check_if_open will return fals after calling this
       '''
        with suppress(KeyError):
            del self.cfg['is_programming_running_info']
        with suppress(FileNotFoundError, AttributeError):
            os.remove(self.check_file)


    def _check_if_open_using_config(self):
        key = 'is_programming_running_info'
        try:
            values = self.cfg[key]
            if os.name == 'posix':
                old_pid = values[0]
            else:
                logging.info(str(values))
                old_pid, hwnd = values
            name, exists = pidutil.process_exists(int(old_pid))
            if exists:
                # Todo, the checks with the name part haven't been tested
                if self.is_installed() and self.pcfg['app_name'] not in name:
                    _raise = True
                elif not self.is_installed() and 'python' not in name.lower():
                    _raise = True
                else:
                    _raise = False

                if _raise:
                    del self.cfg[key]
                    self.cfg.save()
                    logging.debug(
                        'Last time the program ran it didn\'t close properly')
                    raise KeyError
                else:
                    logging.debug('Loading the program as it\'s already running')
                    if os.name == 'posix':
                        pidutil.show_window(self.app_name)
                    if os.name == 'nt':
                        hwnd = pywintypes.HANDLE(int(hwnd))
                        pidutil.show_window(hwnd)
                    logging.info('Exiting new instance of our program')
                sys.exit(800)

        except(KeyError, ValueError):
            # Start program normally
            self.cfg[key] = [os.getpid()]
            self.cfg.save()
            if os.name == 'nt':
                def add_hwnd():
                    hwnd = self.get_windows_hwnd()
                    self.cfg[key].append(self.get_windows_hwnd())
                    self.cfg.save()
                self.root.after(1000, add_hwnd)

    def _check_if_open_using_path(self):
        try:
            with open(self.check_file, "r") as f:
                old_pid = f.readline()
                name, exists = pidutil.process_exists(int(old_pid))
                logging.debug(
                    'name of process gotten ,old_pid : {0} {1}  '.format(
                        name,
                        old_pid))
                if not exists:
                    f.close()
                    os.remove(self.check_file)
                    logging.debug('The old program is not runnning anymore')
                    raise FileNotFoundError
                else:
                    logging.debug(
                        'Loading the program as it\'s already running')
                    if os.name == 'posix':
                        pidutil.show_window(self.app_name)
                    if os.name == 'nt':
                        hwnd = f.readline()
                        logging.debug('hwnd from file ' + str(hwnd))
                        hwnd = pywintypes.HANDLE(int(hwnd))
                        pidutil.show_window(hwnd)
                    logging.info('Exiting new instance of our program')
                sys.exit()
        # Open as Normal
        except FileNotFoundError:
            with open(self.check_file, 'w') as f:
                f.write(str(os.getpid()) + '\n')
                if os.name == 'nt':
                    def add_hwnd():
                        with open(self.check_file, 'a') as f:
                            hwnd = self.get_windows_hwnd()  # FUCK U doesn't run in a function 
                            logging.info('writing hwnd '+ str(hwnd))
                            f.write(str(hwnd) + '\n')
                    self.root.after(2000, add_hwnd)
                set_windows_permissions(self.check_file)

    def get_windows_hwnd(self):
        '''Right now only knows how to get hwnd for a tkinter application,
        if the program is not yet running you will need to use
        the tkinter after method'''
        return int(self.root.wm_frame(), 0)

    def shutdown(self):
        ''' shutdown operations on exit, this is run by
        a finaly statement after the tkinter mainloop ends
        call root.quit to get here,
        Note you will still need to call sys.exit()
        '''
        logging.info('Shutdown procedures being run!')
        for func in self.shutdown_cleanup.values():
            func()
        session_time = round((time.time() - self.start_time)/60, 0)
        logging.info('session time: {0} minutes'.format(session_time))
        logging.info('End..')

    @staticmethod
    def rel_path(*file_path):
        '''
        gives relative paths to files, works when frozen or unfrozen
        to get the root path just pass in the string __file__
        '''
        if hasattr(sys, 'frozen'):
            if file_path[0] == '__file__':
                return sys.executable
            return os.path.join(os.path.dirname(sys.executable), *file_path)
        else:
            try:
                main_file = AppBuilder.main_file
            except AttributeError:
                # The first call setting up the instance
                main_file = inspect.stack()[1][1]
            if file_path[0] == '__file__':
                return os.path.realpath(main_file)
            return os.path.join(os.path.dirname(main_file), *file_path)

def setup_logger(log_file, level=logging.DEBUG):
    '''One function call to set up logging with some nice logs about the machine'''
    cfg = AppBuilder.get_pcfg()
    logger = cfg['log_module']
    # todo make sure structlog is compliant and that logbook is also the correct name???
    assert logger in ("logging", "logbook", "structlog"), 'bad logger specified'
    exec("import {0};logging = {0}".format(logger))

    AppBuilder.logger = logging

    logging.basicConfig(
        filename=log_file,
        filemode='w',
        level=level,
        format='%(asctime)s:%(levelname)s: %(message)s')  # one run
    logging.debug('System is: %s' % platform.platform())
    logging.debug('Python archetecture is: %s' % platform.architecture()[0])
    logging.debug('Machine archetecture is: %s' % platform.machine())

    set_windows_permissions(log_file)


def set_windows_permissions(filename):
    '''
    At least on windows 7 if a file is created on an Admin account,
    Other users will not be given execute or full control.
    However if a user creates the file himself it will work...
    So just always change permissions after creating a file on windows

    Change the permissions for Allusers of the application
    The Everyone Group
    Full access

    http://timgolden.me.uk/python/win32_how_do_i/add-security-to-a-file.html
    '''
    #Todo rename this to allow_all, also make international not just for english..
    if os.name == 'nt':
        try:
            everyone, domain, type = win32security.LookupAccountName(
            "", "Everyone")
        except Exception:
            # Todo fails on non english langauge systesm ... FU WINDOWS
            # Just allow permission for the current user then...
            everyone, domain, type = win32security.LookupAccountName ("", win32api.GetUserName())
        # ~ user, domain, type = win32security.LookupAccountName ("", win32api.GetUserName())
        #~ userx, domain, type = win32security.LookupAccountName ("", "User")
        #~ usery, domain, type = win32security.LookupAccountName ("", "User Y")

        sd = win32security.GetFileSecurity(
            filename,
            win32security.DACL_SECURITY_INFORMATION)
        # instead of dacl = win32security.ACL()
        dacl = sd.GetSecurityDescriptorDacl()

        #~ dacl.AddAccessAllowedAce(win32security.ACL_REVISION, con.FILE_GENERIC_READ | con.FILE_GENERIC_WRITE, everyone)
        #~ dacl.AddAccessAllowedAce(win32security.ACL_REVISION, con.FILE_ALL_ACCESS, user)
        dacl.AddAccessAllowedAce(
            win32security.ACL_REVISION,
            con.FILE_ALL_ACCESS,
            everyone)

        sd.SetSecurityDescriptorDacl(1, dacl, 0)   # may not be necessary
        win32security.SetFileSecurity(
            filename,
            win32security.DACL_SECURITY_INFORMATION,
            sd)


def setup_raven():
    '''we setup sentry to get all stuff from our logs'''
    pcfg = AppBuilder.get_pcfg()
    from raven.handlers.logging import SentryHandler
    from raven import Client
    from raven.conf import setup_logging
    client = Client(pcfg['raven_dsn'])
    handler = SentryHandler(client)
    # TODO VERIFY THIS -> This is the way to do it if you have a paid account, each log call is an event so this isn't going to work for free accounts...
    handler.setLevel(pcfg["raven_loglevel"])
    setup_logging(handler)
    return client





from setuptools import setup, find_packages # Always prefer setuptools over distutils
from os import path
setup(
	name = 'peasoup',
	version='0.1.5',
	description = 'Common tools for application deployment!',
	url = 'https://github.com/timeyyy/apptools',
	author='timothy eichler',
	author_email='tim_eichler@hotmail.com',
	license='BSD3',
	classifiers=[
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Build Tools',
            'License :: OSI Approved :: BSD License',
            'Programming Language :: Python :: 3',
	],
        install_requires = [
            'tkquick',
            'psutil',
            'appdirs',
            'timstools'
        ],

	keywords = 'gui programming apptools python3 permissions deployment application',

	packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
)



import os 
import sys
import subprocess
import logging

from peasoup.util import lazy_import

@lazy_import
def esky():
    import esky
    return esky

class Restarter():
    '''Easily restart script or frozen apps'''

    def start(self):
        self.app_restart()

    @classmethod
    def app_restart(cls):
        '''
        Restart a frozen esky app
        Can also restart if the program is being run as a script
        passes restarted as an argument to the restarted app
        '''
        logging.debug('In restarter')
        executable = sys.executable.split(os.sep)[-1]
        if os.name == 'nt':
            logging.info('executable is is %s' % executable)
            # restart works here when testing compiled version only
            if executable != 'python.exe':
                boot_strap_path = cls.esky_bootstrap_path()
                bootstrap_file = os.path.join(boot_strap_path, executable)
                logging.info('Bootstrap file for restart: %s' % bootstrap_file)

                subprocess.Popen(
                    [os.path.join(bootstrap_file, bootstrap_file), 'restarted'])
        #~ os.execl(bootstrap_file, bootstrap_file, * sys.argv)     # TBD make this work This restarts our process and doesn't return, for somet reason the new restarted version is the old and all fucked


    @staticmethod
    def esky_bootstrap_path():
        '''
        returns the executable path
        '''
        return esky.util.appdir_from_executable(sys.executable)




"""

  peasoup.gui.util:  misc gui utility functions for peasoup

"""


class TkErrorCatcher:

    '''
    In some cases tkinter will only print the traceback.
    Enables the program to catch tkinter errors normally

    To use
    import tkinter
    tkinter.CallWrapper = TkErrorCatcher
    '''

    def __init__(self, func, subst, widget):
        self.func = func
        self.subst = subst
        self.widget = widget

    def __call__(self, *args):
        try:
            if self.subst:
                args = self.subst(*args)
            return self.func(*args)
        except SystemExit as msg:
            raise SystemExit(msg)
        except Exception as err:
            raise err
